// Driver for SAA1064 i2c to multiplexed four-digit, seven-segment led display
// Modified for speden spelit nopeustesti

// SAA1064 write addresses set by connecting pin1 (ADR) to different voltages
#define LED_ADDR0       0x70  // ADR to Vee
#define LED_ADDR1       0x72  // ADR to 3/8 Vcc
#define LED_ADDR2       0x74  // ADR to 5/8 Vcc
#define LED_ADDR3       0x76  // ADR to Vcc

// Control bits
// C0 = 0 = static mode  C0 = 1 dynamic mode
// C1 = 0/1 digits 1 and 3 are blanked/not blanked
// C2 = 0/1 digits 2 and 4 are blanked/not blanked
// C3 = 1 all segments outputs turned on for segment test
// C4 = 1 adds 3ma to segment output
// C5 = 1 adds 6ma to segment output
// C6 = 1 adds 12ma to segment output
// C7 n/a

#define LED_CONTROL 0b01110111  // Dynamic mode 21mA
// Segment table (In hex: d.p.=80,g=40,f=20,e=10,d=08,c=04,b=02,a=01)
byte const seg_table[16]=
{0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,0x7F,0x6F}; // 0,1,2,3,4,5,6,7,8,9
/*
{0b11111001,0b11000000,0b10110011,0b11100011,0b11001010,0b01101011,0b01111011,
 0b11100000,0b11111011,0b11101011,0b11111010,0b01011011,0b00111001,0b11010011,
 0b00111011,0b00111010}; // 0,1,2,3,4,5,6,7,8,9,A,B,C,D,E,F
*/
byte const codeb_table[16]=
{0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,0x7F,0x6F}; // 0,1,2,3,4,5,6,7,8,9
/*
{0b11111001,0b11000000,0b10110011,0b11100011,0b11001010,0b01101011,0b01111011,
 0b11100000,0b11111011,0b11101011,0b00000010,0b00111011,0b11011010,0b00011001,
 0b10111010,0b00000000}; // 0,1,2,3,4,5,6,7,8,9,A,B,C,D,E,F
*/

byte const ascii_table[26]=
{0x77,0x7c,0x39,0x5e,0x79,0x71,0x3d,0x74,0x04,0x0e,0x70,0x38,0x37,
 0x54,0x5c,0x73,0x67,0x50,0x6d,0x78,0x1c,0x1c,0x1c,0x76,0x6e,0x5b};


// Message examples
byte const blank[4] = {0x00,0x00,0x00,0x00};// Blank display
byte const   boot[4] = {0x7c,0x5c,0x5c,0b01111000}; //  bOOt
byte const    on[4] = {0x00,0x00,0x5c,0x54}; //   On
byte const   off[4] = {0x00,0x5c,0x71,0x71}; //  OFF

enum {blank_,boot_,on_,off_};

//--------------------------------------------------------------------------
// Sends number to four-digit led display via SAA1064  (0 to 9999)
void send_saa1064_num(int8 addr,int8 control,int16 num)
{
   int8 j;                       // Digit index
   int8 led_data[4];     // Ram array for segment data
   int16 temp16;
   
   // Get led segments for all four digits. Don't need to divide to get units.
   temp16 = num; 
   j = 0;
 
   while (j <= 3)
   {
       if (j != 0) temp16 /= 10;
       led_data[j] = seg_table[temp16 %10];
       j++;
    }

   // Do leading zero blanking
   if (num<1000) led_data[3] = 0x00;
   if (num<100)  led_data[2] = 0x00;
   if (num<10)   led_data[1] = 0x00;

   // Add decimal point if required
   //led_data[2] |= 0x80;

   // Send led segments
   i2c_start();
   i2c_write(addr);        // Send SAA1064 address
   i2c_write(0x00);        // Send instruction byte. Zero is control reg.
   i2c_write(control);     // Send control byte

   // Auto increment applies for digit data. Have to send out digits in reverse
   // because most significant digit is 1st digit.
   for (j=4;j>=1;j--)           
      i2c_write(led_data[j-1] & 0xff);   
    
   i2c_stop();
}
//---------------------------------------------------------------------------
//Sends message to led display via SAA1064
void send_saa1064_msg(int8 addr,int8 control,int8 msg)
{
   int8 j;
   int8 led_data[4]; // Ram array for segments
   
   // Get segments for message.
   switch (msg)
   {
      case blank_:       
         for(j=0;j<=3;j++)
         led_data[j] = blank[j];
        break;

      case boot_:                 
         for(j=0;j<=3;j++)
         led_data[j] = boot[j];
        break;

      case on_:                 
         for(j=0;j<=3;j++)
         led_data[j] = on[j];
        break;

      case off_:                 
         for(j=0;j<=3;j++)
         led_data[j] = off[j];
         break;

      default:               
      for(j=0;j<=3;j++)
         led_data[j] = blank[j];
         break;
   }

   // Send led segments
   i2c_start();
   i2c_write(addr);      // Send  SAA1064 address
   i2c_write(0x00);     // Send instruction byte. Zero is control reg.
   i2c_write(control);  // Send control byte

   for (j=0;j<=3;j++)
      i2c_write(led_data[j]);
 
   i2c_stop();
} 
//---------------------------------------------------------------------------

void send_saa1064_hex(int8 addr,int8 digit,int8 value)
{
   i2c_start();
   i2c_write(addr);      // Send  SAA1064 address
   i2c_write(digit+1);   // Send instruction byte. Zero is control reg, so digit is 1+control
   i2c_write(seg_table[value]);     // Send segment data
   i2c_stop();
}

void send_saa1064_codeb(int8 addr,int8 digit,int8 value)
{
   i2c_start();
   i2c_write(addr);      // Send  SAA1064 address
   i2c_write(digit+1);   // Send instruction byte. Zero is control reg, so digit is 1+control
   i2c_write(codeb_table[value]);     // Send segment data
   i2c_stop();
} 

void send_saa1064_raw(int8 addr,int8 digit,int8 value)
{
   i2c_start();
   i2c_write(addr);      // Send  SAA1064 address
   i2c_write(digit+1);   // Send instruction byte. Zero is control reg, so digit is 1+control
   i2c_write(value);     // Send segment data
   i2c_stop();
}

//Sends scrolling string to led display via SAA1064
void send_saa1064_string(int8 addr,char* message,int8 speed,int8* new_press)
{
	int8 i,j;
	for (i=0;i<strlen(message)+6;i++) {
		if (*new_press != 0)
			return;
		i2c_start();
		i2c_write(addr);
		i2c_write(1);
		for (j=0;j<4;j++) {
			if (i+j<5||i+j>strlen(message)+4) // blanks before and after message
				i2c_write(0x00);
			else if (message[i+j-5] == 0x20) // space
				i2c_write(0x00);
			else if (message[i+j-5] == '.' || message[i+j-5] == ',') // comma
				i2c_write(0x80);
			else if (message[i+j-5] >= '0' && message[i+j-5] <= '9') // number
				i2c_write(seg_table[message[i+j-5]-'0']);
			else
				i2c_write(ascii_table[message[i+j-5]-'a']);		// (hopefully) letter
		}	
	i2c_stop();
	delay_ms(speed);
	}
}
