////////////////////////////////////////////////////////////
// Speden spelit v. 0.3
// (C) jaka
// 2.10.2013
////////////////////////////////////////////////////////////
// PINOUTS FOR DUAL ATARI VERTICAL PCB:
// X1:                   X2:
// 1 YE BUT  RC0         1 I2C SCL RB6
// 2 GR LED  RC5         2 I2C SDA RB4
// 3 YE LED  RC3         3 RD BUT  RC1
// 4 RD LED  RC6         4         RA3
// 5         RC7/AN9     5
// 6 BUZZER  RC4/P1B     6 BL LED  RC2
// 7 GR BUT  RB7         7 +5V     +5V
// 8 GND     GND         8 GND     GND
// 9 BL BUT  RB5/AN11    9
////////////////////////////////////////////////////////////
// Changes from 0.1 to 0.2
// - Changed duty cycle of green button to remove resonance of speaker
// - Changed startup demo text to show 0.2
// - Added difficult game version to yellow button - starts from 100 points
// - Added memory game to green button. Memory test has separate high score
//
// Changes from 0.2 to 0.3
// - added parameter to saa1064 scrolling function to stop scrolling if button is pressed
// - moved displaying of high scores inside switch-case. now also red button show hiscores
// - fixed memory test bug of not clearing seq_length after game

#include <18F14K50.h>
#DEVICE PASS_STRINGS=IN_RAM // allows passing constant char arrays to saa1064.h function
							// copies string from rom to ram when needed
#include <stdlib.h>			// for random number generation
#include <internal_eeprom.c>	// for reading / writing int16 for highscores
#fuses INTRC_IO,NOWDT,NOPROTECT_0,NOPROTECT_1,NOLVP,NODEBUG,NOBROWNOUT,NOPLLEN,CPUDIV1,PUT,NOMCLR
#use delay(clock=4000000)
#use i2c(master,sda=PIN_B4,scl=PIN_B6,fast=70000) 	// SAA1064 can reliably handle only 70kbps I2C
// #define RAND_MAX 3
#include "saa1064.h"
#define MAXSTACK 10			// maximum number of pending button presses

int8 next=0;
int8 previous = 0;
int8 button_stack[MAXSTACK];
int8 stacklevel = 0;
int8 current_buttons = 4;
int8 last_buttons = 4;
int8 new_press=0;
int8 mode=0;
int1 memorygame=FALSE;

void blue(void);
void yellow(void);
void green(void);
void red(void);
void alloff(void);
void litnext(int allnew);
void litbutton(int color);
void decrease_delay(void);
int check_press(void);
int addtostack(int);


#int_timer1
void timer1isr(void) {
	if (new_press==0) {
		last_buttons=current_buttons;
		current_buttons=5;
		if (!input(pin_b5))	// blue
			current_buttons=0;
		if (!input(pin_c0))	// yellow
			current_buttons=1;
		if (!input(pin_b7))	// green
			current_buttons=2;
		if (!input(pin_c1))	// red
			current_buttons=3;
		if (current_buttons<5 && current_buttons!=last_buttons)
			new_press=1;
	}	
}

int check_press(void) {
	int i=0;
	if (stacklevel>0) {
		if (button_stack[0] == current_buttons) {
			stacklevel--;
			for (i=0;i<stacklevel;i++)
				button_stack[i] = button_stack[i+1];
			return 1;
		} else
			return 0;
	} else	
		return 0;
}		

int addtostack(int button) {
	if (stacklevel<MAXSTACK-1) {
		stacklevel++;
		button_stack[stacklevel-1] = next; // was button, not next
		return 1;
	} else
		return 0;	// stack is full
}		

void blue(void) {
	OUTPUT_HIGH(pin_c3);
	OUTPUT_HIGH(pin_c5);
	OUTPUT_HIGH(pin_c6);
	OUTPUT_LOW(pin_c2);
	setup_timer_2(T2_DIV_BY_16,164,1); // 368 Hz F#4
	set_pwm1_duty(82);
}

void yellow(void) {
	OUTPUT_LOW(pin_c3);
	OUTPUT_HIGH(pin_c5);
	OUTPUT_HIGH(pin_c6);
	OUTPUT_HIGH(pin_c2);
	setup_timer_2(T2_DIV_BY_16,124,1); // 488 Hz B4
//	set_pwm1_duty(62);
	set_pwm1_duty(60);
}

void green(void) {
	OUTPUT_HIGH(pin_c3);
	OUTPUT_LOW(pin_c5);
	OUTPUT_HIGH(pin_c6);
	OUTPUT_HIGH(pin_c2);
	setup_timer_2(T2_DIV_BY_16,102,1); // 588 Hz D5
//	set_pwm1_duty(51);	// original for 50% duty cycle
	set_pwm1_duty(6);	// lower DC to reduce resonance of speaker
}

void red(void) {
	OUTPUT_HIGH(pin_c3);
	OUTPUT_HIGH(pin_c5);
	OUTPUT_LOW(pin_c6);
	OUTPUT_HIGH(pin_c2);
	setup_timer_2(T2_DIV_BY_16,82,1); // 733 Hz F#5
	set_pwm1_duty(41);
}

void alloff(void) {
	OUTPUT_HIGH(pin_c3);
	OUTPUT_HIGH(pin_c5);
	OUTPUT_HIGH(pin_c6);
	OUTPUT_HIGH(pin_c2);
	set_pwm1_duty(0);
}

void litnext(int allnew) {
		next=(rand()%4);
		if (allnew) {
			while(next==previous)
				next=(rand()%4);
			previous=next;
		}
		switch (next) {
			case 0:
				blue();
				break;
			case 1:
				yellow();
				break;
			case 2:
				green();
				break;
			case 3:
				red();
				break;
		}
}

void litbutton(int color) {
		switch (color) {
			case 0:
				blue();
				break;
			case 1:
				yellow();
				break;
			case 2:
				green();
				break;
			case 3:
				red();
				break;
		}
}

int16 decrease_delay(int16 viive) {
	if (viive>7000)
		viive=0.99*viive;
	else
		viive=0.997*viive;
	return viive;
}		

void main(void) {
	int8 i=0;
	int16 pisteet=0;
	int16 viive=16000;
	int16 hiscore=0;
	int16 memhiscore=0;
	int16 memgameseed=0;
	int8 seq_length=0;
	int8 memorybutton=0;
	
	PORT_A_PULLUPS(0b00001000);	// Activates weak pull-up on pin RA3
	PORT_B_PULLUPS(0b11110000);  // Activates weak pull-up on pins RB4-7
	
	delay_ms(500);	// to help with power-up (?)
	if (read_eeprom(0)==0xFF && read_eeprom(1)==0xFF) {	// Reprogramming has erased EEPROM
		write_int16_eeprom(0, 0);					// Set highscore to 0
		write_int16_eeprom(2, 0);					// Set memory game highscore to 0
	}	
	hiscore = read_int16_eeprom(0);
	memhiscore = read_int16_eeprom(2);			// must use even addresses because one int16 takes two bytes

	send_saa1064_msg(LED_ADDR0,LED_CONTROL,blank_);

	
	OUTPUT_LOW(pin_c4);	// needed only to get pic c4 to output fow pwm control
	setup_timer_1(T1_INTERNAL|T1_DIV_BY_1);
	enable_interrupts(INT_TIMER1);
	enable_interrupts(GLOBAL);

	setup_timer_0(RTCC_INTERNAL|RTCC_DIV_64); // set timer0 to maximum of 4s

	// INITIALIZE PWM
	setup_ccp1(CCP_PWM | CCP_PULSE_STEERING_B); // choose P1B for PWM output
	output_high(pin_c5);	// setup_ccp1 leaves P1A driven low..

	send_saa1064_string(LED_ADDR0,"speden spelit nopeustesti versio 0.3",200, &new_press);
	
	mode=6;
	while(1) {
		switch (mode) {		// DEMO
		 case 0:
		 	
			output_float(pin_c4);	// disable buzzer for demo mode
			if (new_press) {
				alloff();
				mode=1;
				srand(get_timer0());
				if (current_buttons==1)	{	// yellow button
					pisteet = 100;			// difficult game - start from 100
					for (i=0; i<pisteet; i++)
						viive = decrease_delay(viive);
				}		
				if (current_buttons==2)	{	// green button
					mode=4;					// memory game
					memgameseed = get_timer0(); // get 16 bit seed for memory game
				}		
				if (current_buttons==3)		// red button
					mode=6;					// show high scores
						
				if (mode != 6) {			// do not show 0 and wait one second if hiscore disp
					send_saa1064_num(LED_ADDR0,LED_CONTROL,pisteet);
					delay_ms(1000);
				}	
				output_low(pin_c4);	// enable buzzer output for game
				new_press=0;
				set_timer0(0);
			}
			
			if (get_timer0()>viive) {
				litnext(1);
				set_timer0(0);
			}
			
			if (get_timer0()>viive*2/3) {
				alloff();
			}
			
			break;
		 
		 case 1:	// GAME
		 	
			if (new_press) {
				// if (1) {
				if (check_press()==1) {
					pisteet++;
					send_saa1064_num(LED_ADDR0,LED_CONTROL,pisteet);
				} else
					mode=2;
				new_press=0;
			}
				
			if (get_timer0()>viive) {
				litnext(1);
				if (addtostack(next)==0)
					mode=2;
				set_timer0(0);
			viive = decrease_delay(viive);
			}
			
			if (get_timer0()>viive*2/3) {
				alloff();
			}
			
			if (mode == 2 && pisteet>hiscore)
				mode=3;
			break;
		case 2:		//GAME OVER
			
				
			for (i=0; i<4; i++) {
				setup_timer_2(T2_DIV_BY_16,150,1); //
				set_pwm1_duty(75);
				OUTPUT_LOW(pin_c3);
				OUTPUT_LOW(pin_c5);
				OUTPUT_LOW(pin_c6);
				OUTPUT_LOW(pin_c2);
				delay_ms(500);
				alloff();
				setup_timer_2(T2_DIV_BY_16,164,1); // 368 Hz F#4
				set_pwm1_duty(82);
				delay_ms(500);
			}	
			alloff();
			mode = 5;
			break;
		case 3:		// HIGH SCORE
			if (memorygame) {
				memhiscore=pisteet;
				write_int16_eeprom(2, memhiscore);
			}
			else {	
				hiscore=pisteet;
				write_int16_eeprom(0, hiscore);
			}	
			for (i=0; i<4; i++) {
				setup_timer_2(T2_DIV_BY_16,110,1);
//				set_pwm1_duty(55);
				set_pwm1_duty(6);
				OUTPUT_LOW(pin_c3);
				OUTPUT_LOW(pin_c5);
				OUTPUT_LOW(pin_c6);
				OUTPUT_LOW(pin_c2);
				delay_ms(300);
				alloff();
				setup_timer_2(T2_DIV_BY_16,100,1);
//				set_pwm1_duty(50);
				set_pwm1_duty(6);
				delay_ms(300);
				send_saa1064_msg(LED_ADDR0,LED_CONTROL,blank_);
				setup_timer_2(T2_DIV_BY_16,90,1);
//				set_pwm1_duty(45);
				set_pwm1_duty(6);
				delay_ms(300);
				send_saa1064_num(LED_ADDR0,LED_CONTROL,pisteet);
			}	
			alloff();
			send_saa1064_string(LED_ADDR0,"congratulations new high score",200, &new_press);
			send_saa1064_num(LED_ADDR0,LED_CONTROL,pisteet);
			mode = 5;
			break;			

		 case 4:	// MEMORY GAME
		 	memorygame = TRUE;
		 	srand(memgameseed);
		 	for (i=0; i<=seq_length; i++) {
				litnext(0);
		 		delay_ms(250);
		 		alloff();
		 		delay_ms(150);
			}
			
			srand(memgameseed);
			for (i=0; i<=seq_length; i++) {
				while (!new_press) {
				}
				memorybutton = (rand()%4);
				new_press=0;
				if (current_buttons == memorybutton) {
					pisteet++;
					litbutton(memorybutton);
					send_saa1064_num(LED_ADDR0,LED_CONTROL,pisteet);
					delay_ms(100);
					alloff();
				} else {
					if (pisteet>memhiscore)
						mode=3;
					else
						mode=2;
					break;
				}	
			}
			if (mode == 4)
				delay_ms(1000);
			seq_length++;		
			break;

		 case 5:	// INIT
			new_press=0;
			pisteet=0;
			viive=16000;
			stacklevel=0;
			mode=0;
			memorygame=FALSE;
			seq_length=0;
			break;

		 case 6:	// DISPLAY HIGH SCORES DEMO
			send_saa1064_string(LED_ADDR0,"speed test high score",200, &new_press);
			send_saa1064_num(LED_ADDR0,LED_CONTROL,hiscore);
			if (new_press == 0)
				delay_ms(1000);
			send_saa1064_string(LED_ADDR0,"memory test high score",200, &new_press);
			send_saa1064_num(LED_ADDR0,LED_CONTROL,memhiscore);
			if (new_press == 0)
				delay_ms(1000);
			send_saa1064_string(LED_ADDR0,"hit blue button to start easy speed test",200, &new_press);
			send_saa1064_string(LED_ADDR0,"hit yellow button to start hard speed test",200, &new_press);
			send_saa1064_string(LED_ADDR0,"hit green button to start memory test",200, &new_press);
			send_saa1064_string(LED_ADDR0,"hit red button to display high scores",200, &new_press);
			mode=0;
			break;


		}	// END OF SWITCH
			
	}	// END OF ENDLESS LOOP
}	// END OF MAIN